# We have to import display.display in this way for mocks to work
import IPython.display
from IPython.display import Image

from poked.client import get_pokemon


def get_sprite(ID: int) -> Image:
    """Get the sprite for the given Pokemon ID"""
    url = f"https://raw.githubusercontent.com/PokeAPI/sprites/master/sprites/pokemon/{ID}.png"
    return Image(url=url, width=200, embed=False)


# Does not return, uses display() as a side effect
async def show(name: str) -> None:
    """Show the sprite for the given Pokemon name, asynchronously"""
    pokemon = await get_pokemon(name)
    # The ID is the index, which on a series is the property called name. Make sure to convert it to an int
    ID: int = pokemon.name  # type: ignore

    if ID is None:
        raise ValueError(f"Could not find Pokemon with name {name}")

    # type: ignore
    show_by_id(ID)


def show_by_id(ID: int):
    """Show the sprite for the given Pokemon ID"""
    IPython.display.display(get_sprite(ID))



# Rely on appdirs to set up a cache for the poked app
#
# The cache is stored in the user's cache directory, which is
# platform-specific.  On Linux, it's in ~/.cache/poked. On MacOS
# it's in ~/Library/Caches/poked. On Windows it's in %LOCALAPPDATA%\poked.
#
# That also depends on other env vars, so see the appdirs docs for more info.
#
# The cache is a directory containing a file for each query, named
# after the query's hash.  The file contains the query's result, in
# JSON format.

import hashlib
import json
import os

import appdirs

CACHE_DIR = "poked"


def get_cache_filename(query):
    """Return the filename for the given query in the cache directory"""
    # Hash the query to get a unique filename
    hash = hashlib.sha256(query.encode("utf-8")).hexdigest()
    return os.path.join(appdirs.user_cache_dir(CACHE_DIR), hash)


def get_cached_query(query):
    """Return the cached result for the given query, or None if not cached"""
    filename = get_cache_filename(query)
    if not os.path.exists(filename):
        return None

    with open(filename, "r") as f:
        # Attempt to load the JSON and return it.
        # If it fails, we must clear it from the cache.
        try:
            return json.load(f)
        except json.decoder.JSONDecodeError:
            os.remove(filename)
            return None


def cache_query(query, result):
    """Cache the result for the given query"""
    filename = get_cache_filename(query)
    os.makedirs(appdirs.user_cache_dir(CACHE_DIR), exist_ok=True)
    with open(filename, "w") as f:
        json.dump(result, f)


def clear_cache(query=None):
    """Clear the cache, or just the given query if specified"""
    if query is None:
        os.remove(appdirs.user_cache_dir(CACHE_DIR))
    else:
        filename = get_cache_filename(query)
        if os.path.exists(filename):
            os.remove(filename)
